@@ -30,8 +30,8 @@
 #include "eel-debug.h"
 #include "eel-lib-self-check-functions.h"
 #include "eel-string.h"
+#include "eel-i18n.h"
 #include <glib-object.h>
-#include <libgnome/gnome-i18n.h>
 #include <math.h>
 #include <stdlib.h>
 #include <sys/time.h>
